/***********************************************************

	Command:  DDNEW -- Create new work drawing file

	Purpose:  This program is used to provide a direct [New]
		  menu function, which is supposed to create a
		  new empty drawing file. Originally, the [New]
		  menu function was written with inline TCL program
		  body in the menu file, which is quite simple and
		  primitive in comparision to this program.

	Programmed by Kang, Hsin-Min.
	Released at Feb 14, 1998
	Revised at  Apr 22, 1998 -- Include Japenese Text in SJIS codes

	Copyright(c) 1998, TCAM Development House, Taipei.
	All rights Reserved.

	Status: TwinCAD Standard Command, with full support from
		TCAM Development House.

************************************************************/
int	IsWin;
int	nSupport=identify("wnd_resize");

int	nMain[6]={2,11,8,72,0x3270,0x8000};
int	widMain;	// Main Dialog Window ID
int	nTatrb=0x2f0;
int	nSatrb=0x270;

int	bWname; 	// Name of new work file
int	bPname; 	// Name of Proto file
int	bProto; 	// Proto
int	bCancel;	// Cancel key
int	bCreate;	// Create key
int	bHelp;		// Help key
int	bNomore;	// No more popup

char	szBtn[10][10];	// For all common buttons (8 chars)
char	szText[25][48]; // Our display text buffer, see SetMsgText()
			// at the end of this file
#define USEPROTO	1
#define NODIALOG	2
#define SETFILE 	"NEW.SET"
int	nUseProto=0;
char	szProtoFile[128]="";
char	szFilename[128]="";

COMMAND DDNew()
{
	int	nBtn;
	char	szTmp[128];
	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}
	SetMsgText();
	Initialize();
	if ( nUseProto & NODIALOG ) {	// Do it directly
		if ( CheckSave() )  exit();	// The use can quit it here
		if ( AskNewFile() ) {
			CreateFile();
			exit();
		}
		// The operator must have quited the file window, so
		// it is a chance to re-enter the dialog again for
		// modification and confirmation.
	}
	OpenDialog();
	UpdateDialog();
	for(;;) {
		nBtn = wnd_gbtn(0);
		switch(nBtn) {
		  case -3:
		  case -2:
		  case -1:
		  case	0:
			CloseDialog();
			return 0;
		  case bWname:
			if ( AskNewFile() ) {
				UpdateDialog();
				if ( identify("wnd_state") )
					wnd_state(bCreate,1,0);
			}
			continue;
		  case bPname:
			szTmp = getfile(szProtoFile,"WRK",0x4082,szText[5]);
				/*"Select a workfile for prototype file"*/
			if ( szTmp != "" ) {
				szProtoFile = szTmp;
				UpdateDialog();
			}
			continue;
		  case bProto:
			nUseProto ^= USEPROTO;
			wnd_fmark(bProto,0,(nUseProto&USEPROTO)?1:0);
			continue;
		  case bNomore:
			nUseProto ^= NODIALOG;
			wnd_fmark(bNomore,0,(nUseProto&NODIALOG)?1:0);
			continue;
		  case bCancel:
			CloseDialog();
			return 0;
		  case bCreate:
			if ( szFilename == "" ) {
				beep(); 	// For DOS
				continue;
			}
			if ( CheckSave() )  continue;
			CloseDialog();
			CreateFile();
			return 0;
		  case bHelp:
			show_help("","DDNEW");
			continue;
		  default:
			continue;
		}
	}
}
//
//	Ask for new drawing file to create
//
int	AskNewFile()
{
	char	szTmp[128];
	szTmp = getfile(szProtoFile,"WRK;*.DWG",0x4083,szText[6]);
	       /* "Enter new drawing filename to create" */
	if ( szTmp == "" ) return 0;
	szFilename = szTmp;
	return 1;
}
//
//	Check if the current drawing is modified and ask for saving
//	current drawing file before creating new one.
//
int	CheckSave()
{
	int	nBtn;
	char szOldName[124]=@CWORKFILE;
	if ( !(@workstatus & 1) ) return 0;
	nBtn = wnd2btn(szText[7]/*"Current Drawing Has Been Modified!"*/,
	       szText[8]/*"Do you want to save it first ?"*/,
	       szBtn[3],szBtn[4]);	// Yes, No
	switch(nBtn) {
	  case 0:	break;
	  case 1:	// Yes
		command("SAVE *");
		if ( @CWORKFILE == "" )
			return 1;	// No saving at all
		break;
	  default:	// Cancel
		return 1;
	}
	return 0;
}
//
//	Create new drawing file with/without loading prototype drawing
//
int	CreateFile()
{
	int	cmdecho=@cmdecho;
	@cmdecho = 0;	    // Reset it
	if ( nUseProto & USEPROTO ) {
		command("New L @!\@szProtoFile");
		command("New S @!\@szFilename");
	}
	else {	// TwinCAD will load its default prototype file automatically
		command("New C @!\@szFilename");
		command("ERASE ALL   ");        // Erase everything
		command("PURGE ALL   ");        // Purge everything
	}
	@cmdecho = cmdecho;
	printf(szText[9]/*"\nNew File: %s"*/,szFilename);
}

#define GENERAL 	"General Setup"
#define USEPROTOFILE	"UsePrototypeFile"      /* Priviate to DDNEW */
#define PROTOFILE	"PrototypeWrkFile"
//
Initialize()
{
	int	hdl;
	if ( IsWin) {
		nUseProto = eval(get_profile(GENERAL,USEPROTOFILE,0,"1")+"+0");
		szProtoFile = get_profile(GENERAL,PROTOFILE,0,"TCAD.WRK");
	}
	else {	// Well, if DOS version, we save it to current directory
		hdl = fopen(SETFILE,"r");
		if ( hdl >= 0 ) {
		    fget(hdl,&nUseProto,szProtoFile);
		    fclose(hdl);
		}
	}
	szProtoFile = fsearch(szProtoFile);
	if ( szProtoFile == "" ) {
		if ( nUseProto & USEPROTO )
			nUseProto = 0;
	}
}
//
//	OpenDialog() -- Open main dialog window, create screen items.
//
int	nFsize;
void	OpenDialog()
{
	int	rBase, cBase, col;
	int	rInc=0x0401;
	int	nFwidth;

	nFwidth=(nMain[3]-nMain[2])-4;

	widMain = wndopen(nMain,szText[0]/*"Create New Drawing..."*/,0x3f1f,nMain[5]);
	rBase = 0x0001;
	cBase = 2;
	WndGroup(rBase-1,cBase,nFwidth,1,"",nSatrb);
	nFsize = nFwidth - len(szText[1]);
	bWname = WndEntry(rBase,cBase,szText[1]/*"  Filename: "*/,nFsize|0x100,0);
	rBase += 0x0801;
	bProto = WndCheckBox(rBase,cBase,szText[3]/*"Use Prototype Drawing"*/,nSatrb);
	bNomore = WndCheckBox(rBase,cBase+nFwidth/2,szText[4]/*"Don't pop this dialog again"*/,nSatrb);
	rBase += 0x0801;
	WndGroup(rBase-1,cBase,nFwidth,1,"",nSatrb);
	bPname = WndEntry(rBase,cBase,szText[2]/*" Prototype: "*/,nFsize|0x100,0);
	rBase += 0x0802;
	cBase = nFwidth-30;
	if ( !IsWin )
		cBase += 12;
	bCreate = WndButton(rBase,cBase,0x108,szBtn[0]/*"&OK"*/);
	bCancel = WndButton(rBase,cBase+12,0x108,szBtn[1]/*"&Cancel"*/);
    if ( IsWin )	// Only in Windows version
	bHelp	= WndButton(rBase,cBase+24,0x108,szBtn[2]/*"&Help"*/);
	if ( identify("wnd_state") )
		wnd_state(bCreate,1,1);
}
//
//	Note:	The use of wnd_fmark to make a check on the checkbox
//		is not fully compatible with DOS version, although
//		the expression: wnd_fmark(n,0,(Check)?0x8000:0x80ff)
//		still works.
//
void	UpdateDialog()
{
	wnd_fmark(bProto,0,(nUseProto&USEPROTO)?1:0);
	wnd_fmark(bNomore,0,(nUseProto&NODIALOG)?1:0);
	wnd_ftext(bPname,prints("%-"+prints("%d",nFsize)+"s",szProtoFile),0x2f0);
	wnd_ftext(bWname,prints("%-"+prints("%d",nFsize)+"s",szFilename),0x2f0);
}
//
//	CloseDialog() -- Close the main dialog window, save window attribute
//
void	CloseDialog()
{
	wndclose();
	if ( IsWin) {
		set_profile(GENERAL,PROTOFILE,szProtoFile);
		set_profile(GENERAL,USEPROTOFILE,prints("%d",nUseProto));
	}
	else {
		hdl = fopen(SETFILE,"w");
		if ( hdl >= 0 ) {
		    fput(hdl,&nUseProto,szProtoFile);
		    fclose(hdl);
		}
	}
}
//
//	WndGroup() -- Create Groupping window effect
//
void	WndGroup(int r,int c,int width, int height, char text[], int color)
{
	if ( IsWin )
		wnd_group(r,c,width,height,0x270,text,color|0x300);
	else {
		wnd_loct(r,c);
		wnd_rloct(height,0);
		wnd_field(0,((color&0xff)^0x08)|0x200,(height<<8)|width);
		wnd_itemno(1,-1);
		wnd_rloct(-height,2);
		wnd_rloct(0xfc00,0);
		put3Dstr(text,color);
		wnd_loct(r,c);	      // Reset row offset, fix DOS version bug
	}
}
//
//	put3Dstr() -- Drawing 3D text
//
void	put3Dstr(char str[],int atrb)
{
	int	nLen;
	if ( nSupport ) {
		wnd_puts(str,atrb|0x300);
		return;
	}
	nLen=len(str);
	if ( nLen ) {
		wnd_rloct(0x100,IsWin?0x100:0);
		wnd_puts(str,(atrb^0x0f));
		wnd_rloct(0xff00,-nLen);
		wnd_puts(str,(atrb&0xff0f)|0x8000);
	}
}
//
//	WndCheckBox() -- Create window check box
//
int	WndCheckBox(int r,int c,char text[],int atrb)
{
	int	bID;
	if ( nSupport )
		return wnd_rcbox(r,c,text,0);
	wnd_loct(r,c);
	bID = wnd_field(1,0x1f0,0x102);
	wnd_rloct(0,2);
	if ( IsWin )  atrb |= 0x800;
	wnd_puts(text,atrb);
	return bID;
}
//
//	WndRadioBox() -- Create window radio box
//
int	WndRadioBox(int r,int c,char text[],int atrb)
{
	int	bID;
	if ( nSupport )
		return wnd_rcbox(r,c,text,0x8000);
	wnd_loct(r,c);
	bID = wnd_field(1,0x1f0,0x102);
	wnd_rloct(0,2);
	if ( IsWin ) atrb |= 0x800;
	wnd_puts(text,atrb);
	wnd_fmark(bID,1,IsWin?1:0x8000);     // Force to change state
	wnd_fmark(bID,1,IsWin?0:0x80ff);
	return bID;
}
//
#define OBM_COMBO	"#32738"
int	WndEntry(int row, int col, char str[],int fsize,int combo)
{
	int	bID;

	wnd_loct(row,col);
	put3Dstr(str,0x270);
	bID = wnd_field(0,IsWin?0x1f0:0x21f0,fsize);
	if ( combo == 1 ) {
	    wnd_rloct(0xfe00,0x0600|(fsize&0xff));
	    if ( IsWin) wnd_runit(12);
	    if ( identify("cb_empty") ) {   // Supported only after this version
		 wnd_icon(1,0xA77,0x101,OBM_COMBO);
		 wnd_runit(-1);
		 wnd_rloct(0x0200,0x0002);  // locate forward
	    }
	    else {
		 wnd_field(0,IsWin?0x277:0x370,0x101);
		 wnd_runit(IsWin?-1:16);
		 wnd_ftext(bID+1,"",IsWin?0x277:0x270);
		 wnd_rloct(0x0200,0x0201);  // locate forward
	    }
	}
	else if ( combo == 2 ) {
	    wnd_rloct(0xf400,0x0600|(fsize&0xff));
	    wnd_runit(6);
	    wnd_field(0,IsWin?0x277:0x370,0x101);
	    wnd_ftext(bID+1,"",IsWin?0x277:0x270);
	    wnd_rloct(0x0e00,0xff);
	    wnd_field(0,IsWin?0x277:0x370,0x101);
	    wnd_ftext(bID+2,"",IsWin?0x277:0x270);
	    wnd_runit(-1);
	    wnd_rloct(0,2);  // locate forward
	}
	return bID;
}
//
int	WndButton(int r,int c, int bsize, char text[])
{
	int	i;
	if ( nSupport ) 	// Having direct support ?
		return wnd_button(r,c,bsize,0x1277,text);
	wnd_loct(r,c);
	i = wnd_field(0,IsWin?0x1277:0x370,bsize);
	wnd_puts(text,0x270);
	return i;
}
//
int	set_profile(char section[], char entry[], char value[])
{
	if ( get_profile(section,entry) != value ) {
		put_profile(section,entry,value);
		return 1;
	}
	return 0;
}

char	szAsciiText[][48]={	    // Default English Display Text
/*  0 */   "Create New Drawing...",
/*  1 */   "  Filename: ",
/*  2 */   " Prototype: ",
/*  3 */   "Use Prototype Drawing",
/*  4 */   "Don't show this dialog again",
/*  5 */   "Select a work drawing for prototype file",
/*  6 */   "Enter new drawing filename to create",
/*  7 */   "Current Drawing Has Been Modified!",
/*  8 */   "Do you want to save it first ?",
/*  9 */   "\nNew File: %s",
};

char	szBig5Text[][48]={	   // Default Chinese Big-5 Display Text
/*  0 */   "Ыطsu@...",
/*  1 */   "  sɦW: ",
/*  2 */   "  Wd: ",
/*  3 */   "ϥγWdɧ@Z",
/*  4 */   "nAܳoӳ]wާ@e",
/*  5 */   "ЫwsϨϥΪWd",
/*  6 */   "пJnЫتsɦW",
/*  7 */   "ثeɤewQʹL!",
/*  8 */   "znnNثeɦsɩO ?",
/*  9 */   "\nsP: %s",
};

char	szGB2312Text[][48]={	// Default Chinese GB2312 Display Text
/*  0 */   "µĹͼ...",
/*  1 */   "  ͼ: ",
/*  2 */   "  淶ͼ: ",
/*  3 */   "ʹù淶ͼΪ׸",
/*  4 */   "Ҫʾ趨",
/*  5 */   "ָͼʹõĹ淶ͼ",
/*  6 */   "Ҫͼ",
/*  7 */   "Ŀǰͼѱ!",
/*  8 */   "ҪҪȽĿǰͼ浵 ?",
/*  9 */   "\n±ͼ: %s",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][48]={    // Default Japenese SJIS Display Text
/*  0 */   "V}ʂ쐬",
/*  1 */   "t@C: ",
/*  2 */   "ftHg: ",
/*  3 */   "ftHg̐}ʂg܂B",
/*  4 */   "̕\\\\܂B",
/*  5 */   "ftHĝ߂Ƀ[N̐}ʂIĉB",
/*  6 */   "V}ʃt@C͂ĂB",
/*  7 */   "̐}ʂ͕ύXĂ܂B",
/*  8 */   "O̐}ʃt@Cۑ܂B",
/*  9 */   "\nVt@C: %s",
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][48]={	// Default Korean Display Text
/*  0 */   "Create New Drawing...",
/*  1 */   "  Filename: ",
/*  2 */   " Prototype: ",
/*  3 */   "Use Prototype Drawing",
/*  4 */   "Don't show this dialog again",
/*  5 */   "Select a work drawing for prototype file",
/*  6 */   "Enter new drawing filename to create",
/*  7 */   "Current Drawing Has Been Modified!",
/*  8 */   "Do you want to save it first ?",
/*  9 */   "\nNew File: %s",
};

char	szAsciiBtn[][10]={	   // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "&Cancel",
/*  2 */   "&Help",
/*  3 */   "&Yes",
/*  4 */   "&No",
/*  5 */   "&Create",
};

char	szBig5Btn[][10]={	  // Default English Display Text
/*  0 */   "Tw[&O]",
/*  1 */   "[&C]",
/*  2 */   "[&H]",
/*  3 */   "O[&Y]",
/*  4 */   "n[&N]",
/*  5 */   "Ы[&C]",
};

char	szGB2312Btn[][10]={	    // Default English Display Text
/*  0 */   "ȷ[&O]",
/*  1 */   "ȡ[&C]",
/*  2 */   "˵[&H]",
/*  3 */   "ǵ[&Y]",
/*  4 */   "Ҫ[&N]",
/*  5 */   "[&C]",
};

char	szSJISBtn[][10]={	  // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "ݾ[&C]",
/*  5 */   "[&H]",
/*  3 */   "͂[&Y]",
/*  4 */   "[&N]",
/*  5 */   "쐬[&C]",
};

char	szKoreanBtn[][10]={	    // Default English Display Text
/*  0 */   "&OK",
/*  1 */   "&Cancel",
/*  2 */   "&Help",
/*  3 */   "&Yes",
/*  4 */   "&No",
/*  5 */   "&Create",
};

char	szDOSAsciiBtn[][10]={	      // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   " Cancel ",
/*  2 */   "  Help  ",
/*  3 */   "  Yes   ",
/*  4 */   "   No   ",
/*  5 */   " Create ",
};

char	szDOSBig5Btn[][10]={	     // Default English Display Text
/*  0 */   "  Tw  ",
/*  1 */   "    ",
/*  2 */   "    ",
/*  3 */   "  O  ",
/*  4 */   "  n  ",
/*  5 */   "  Ы  ",
};

char	szDOSGB2312Btn[][10]={	       // Default English Display Text
/*  0 */   "  ȷ  ",
/*  1 */   "  ȡ  ",
/*  2 */   "  ˵  ",
/*  3 */   "  ǵ  ",
/*  4 */   "  Ҫ  ",
/*  5 */   "    ",
};

char	szDOSSJISBtn[][10]={	     // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   "  ݾ ",
/*  2 */   " wv ",
/*  3 */   "  ͂  ",
/*  4 */   "  ",
/*  5 */   "  쐬  ",
};

char	szDOSKoreanBtn[][10]={	       // Default English Display Text
/*  0 */   "   OK   ",
/*  1 */   " Cancel ",
/*  2 */   "  Help  ",
/*  3 */   "  Yes   ",
/*  4 */   "   No   ",
/*  5 */   " Create ",
};

#define BIG5		0x404
#define GB2312		0x804
#define SJIS		0x411
#define KOREAN		0x412
#define ASCII		0x409

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

int	SetMsgText()
{
	char	szBuf[80];
	int	switched;

	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}

	if ( IsWin ) {
	    szBuf = get_profile("General Setup","AlternateLanguage");
	    switched = 0;
	    if ( szBuf!="" )
		switched = eval(szBuf);
	    setdata(szText,szAsciiText);
	    setdata(szBtn,szAsciiBtn);
	    if ( switched ) {
		switch(os_language()) {
		  case BIG5:	setdata(szText,szBig5Text);	break;
		  case GB2312:	setdata(szText,szGB2312Text);	break;
		  case SJIS:	setdata(szText,szSJISText);	break;
		  case KOREAN:	setdata(szText,szKoreanText);	break;
		  default:	setdata(szText,szAsciiText);	break;
		}
		switch(os_language()) {
		  case BIG5:	setdata(szBtn,szBig5Btn);     break;
		  case GB2312:	setdata(szBtn,szGB2312Btn);   break;
		  case SJIS:	setdata(szBtn,szSJISBtn);     break;
		  case KOREAN:	setdata(szBtn,szKoreanBtn);   break;
		  default:	setdata(szBtn,szAsciiBtn);    break;
		}
	    }
	}
	else {	// Switched or not, depends on TGF type in DOS version
	    setdata(szText,szAsciiText);
	    setdata(szBtn,szDOSAsciiBtn);
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szText,szBig5Text);     break;
	      case TGGB2312:  setdata(szText,szGB2312Text);   break;
	      case TGSJIS:    setdata(szText,szSJISText);     break;
	      case TGKOREAN:  setdata(szText,szKoreanText);   break;
	      default:	      setdata(szText,szAsciiText);    break;
	    }
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szBtn,szDOSBig5Btn);     break;
	      case TGGB2312:  setdata(szBtn,szDOSGB2312Btn);   break;
	      case TGSJIS:    setdata(szBtn,szDOSSJISBtn);     break;
	      case TGKOREAN:  setdata(szBtn,szDOSKoreanBtn);   break;
	      default:	      setdata(szBtn,szDOSAsciiBtn);    break;
	    }
	}
	return 0;
}
//
//	End of this Program file
//
